﻿/// <typ>StorageHelperType</typ>
/// <file>StorageHelperType.AttribMethods.cs</file>
/// <create>22.09.2011 00:12:09</crate>
/// <author>Wolfgang Stelzhammer</author>
///
/// <summary>
///
/// </summary>
/// <copyright>Copyright © 2008-2011 Wolfgang Stelzhammer, All rights reserved!</copyright>
using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Reflection;
using System.Text;
using StorageHelper.StorageAttributes;
using StorageHelper.StorageLib;

namespace StorageHelper.StorageBase
{
		public partial class StorageHelperType
		{
				private static int getIdDigitsAttribute()
				{
						Type t = TypeInfo.TypeReflection;

						Contract.Requires(t != null);

						CustomAttibute<IdDigitsAttribute> temp = new CustomAttibute<IdDigitsAttribute>(t);

						if (temp.Count <= 0)
								return StorageLib.TypeInfo._IDIGITS;

						return temp.First<IdDigitsAttribute>().Digits;
				}

				private static List<string> getExcluded()
				{
						Type t = TypeInfo.TypeReflection;

						List<string> ret = new List<string>();

						CustomAttibute<ExcludeAttribute> test = new CustomAttibute<ExcludeAttribute>(t);

						foreach (PropertyInfo prop in test.FoundProperties)
						{
								ret.Add(prop.Name);
						}

						PropertyInfo[] PI = t.GetProperties();
						foreach (PropertyInfo propInfo in PI)
						{
								if (!ret.Contains(propInfo.Name))
								{
										if (!propInfo.CanWrite)
												ret.Add(propInfo.Name);
										else if (propInfo.GetSetMethod() == null)
												ret.Add(propInfo.Name);
								}
						}

						return ret;
				}

				private static Dictionary<string, PropertyInfo> getProperties()
				{
						Type t = TypeInfo.TypeReflection;

						Dictionary<string, PropertyInfo> ret = new Dictionary<string, PropertyInfo>();

						if (TypeInfo.ExcludedProperties == null)
								TypeInfo.ExcludedProperties = getExcluded();

						PropertyInfo[] PI = t.GetProperties();
						foreach (PropertyInfo propInfo in PI)
						{
								if (!TypeInfo.ExcludedProperties.Contains(propInfo.Name))
								{
										ret.Add(propInfo.Name, propInfo);
								}
						}

						return ret;
				}

				private static string getConnectionString()
				{
						Type t = TypeInfo.TypeReflection;

						string ret = string.Empty;

						CustomAttibute<ConnectionStringAttribute> temp = new CustomAttibute<ConnectionStringAttribute>(t);

						if (temp.Count > 0)
								ret = temp.First<ConnectionStringAttribute>().Value;

						if (ret == string.Empty)
								throw new ConnectionStringNotExistingException(_ConnectionString: ret);
						else
								return ret;
				}

				private static StorageEngine getStorageEngine()
				{
						Type t = TypeInfo.TypeReflection;

						CustomAttibute<StorageEngineAttribute> temp = new CustomAttibute<StorageEngineAttribute>(t);

						if (temp.Count > 0)
								return temp.First<StorageEngineAttribute>().Value;
						else
								return null;
				}

				private static bool getAutosave()
				{
						Type t = TypeInfo.TypeReflection;

						CustomAttibute<AutoSaveAttribute> temp = new CustomAttibute<AutoSaveAttribute>(t);
						if (temp.Count == 1)
								return temp[0].Value;
						else
								return false;
				}

				private static string getTypeName()
				{
						Type t = TypeInfo.TypeReflection;
						CustomAttibute<TypeNameAttribute> temp = new CustomAttibute<TypeNameAttribute>(t);

						if (temp.Count == 1)
								return temp[0].Value;
						else
								return t.Name;
				}

				private static string getTypeCollectionName()
				{
						Type t = TypeInfo.TypeReflection;
						CustomAttibute<TypeCollectionNameAttribute> temp = new CustomAttibute<TypeCollectionNameAttribute>(t);

						if (temp.Count == 1)
								return temp[0].Value;
						else
								return TypeInfo.TypeName + "Collection";
				}

				private static StaticTypeInfo getStaticTypeInfo()
				{
						Type t = TypeInfo.TypeReflection;

						CustomAttibute<RuntimeTypeInfoAttribute> temp = new CustomAttibute<RuntimeTypeInfoAttribute>(t, Bindings: BindingFlags.NonPublic | BindingFlags.Static);

						if (temp.Count <= 0 || temp.FoundFields.Count <= 0)
								return new StaticTypeInfo();
						else
						{
								FieldInfo Sti = temp.FoundFields.First<FieldInfo>();
								return (StaticTypeInfo)Sti.GetValue(t);
						}
				}

				private static Dictionary<string, string> getUniqIDs()
				{
						Type t = TypeInfo.TypeReflection;

						Dictionary<string, string> ret = new Dictionary<string, string>();

						CustomAttibute<UniqIDAttribute> temp = new CustomAttibute<UniqIDAttribute>(t);
						for (int i = 0; i < temp.Count; i++)
						{
								string PropName = temp.FoundProperties[i].Name;
								string Value = temp[i].Value;

								ret[PropName] = StaticHelpers.detectBöseUniqId(Value);
						}

						return ret;
				}
		}
}