﻿/// <typ>StorageHelperType</typ>
/// <file>StorageHelperType.Construktors.cs</file>
/// <create>22.09.2011 00:11:30</crate>
/// <author>Wolfgang Stelzhammer</author>
///
/// <summary>
///
/// </summary>
/// <copyright>Copyright © 2008-2011 Wolfgang Stelzhammer, All rights reserved!</copyright>
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using StorageHelper.StorageEngines;
using StorageHelper.StorageLib;

namespace StorageHelper.StorageBase
{
		public abstract partial class StorageHelperType
		{
				public StorageHelperType()
				{
						InitStatic(this.GetType());

						Init();
						RegisterEvents();
						UpdatePropertyContent();
				}

				private void Init()
				{
						//x Note: Engine auf TypeInfo verschieben und einen Statischen Zugang mittels Übergabe von StateInfo per eigenen EventArg
						// Note: Engine wurde auf Statischen teil verschoben.
				}

				private static bool UpdateContentInProgress = false;

				private void RegisterEvents()
				{
						PropertyChanged += UpdateStickyTrue;
						PropertyChanged += UpdateContent;
						PropertyChanged += UpdateUniqID;
						PropertyChanged += DoAutoSave;

						SaveEvent += TypeInfo.Engine.Save;
						SaveEvent += UpdateStickyFalse;

						DeleteEvent += TypeInfo.Engine.Delete;

						ClearEvent += ClearThisObjectHandler;
				}

				private void ClearThisObjectHandler(StorageHelperType sender, StorageHelperEventArgs e)
				{
				}

				private void UpdateUniqID(object sender, PropertyChangedEventArgs e)
				{
						if (e.PropertyName == "Id" || TypeInfo.UniqIDs.ContainsKey(e.PropertyName))
						{
								StateInfo.UniqID = string.Empty;
								List<string> temp = new List<string>();
								temp.Add(IdString);

								foreach (KeyValuePair<string, string> item in TypeInfo.UniqIDs)
								{
										//if (StateInfo.PropertiesContent.ContainsKey(item.Value))
										temp.Add(item.Value + StateInfo.PropertiesContent[item.Key]);
										//else
										//  temp.Add(item.Value + "null");
								}
								StateInfo.UniqID = string.Join("_", temp);
						}
				}

				private void UpdateStickyFalse(StorageHelperType sender, StorageHelperEventArgs e)
				{
						StateInfo.isSticky = true;
				}

				private bool inAutosave = false;

				private void DoAutoSave(object sender, PropertyChangedEventArgs e)
				{
						if (TypeInfo.AutoSave && !inAutosave)
						{
								inAutosave = true;
								while (UpdateContentInProgress)
								{
								}
								this.Save();
								StateInfo.isSticky = false;
								inAutosave = false;
						}
				}

				private void UpdateContent(object sender, PropertyChangedEventArgs e)
				{
						UpdateContentInProgress = true;
						UpdatePropertyContent(e.PropertyName);
						UpdateContentInProgress = false;
				}

				private void UpdatePropertyContent(string PropName = null)
				{
						if (PropName == null || PropName == string.Empty)
						{
								foreach (KeyValuePair<string, PropertyInfo> info in TypeInfo.PropertyInfo)
								{
										string ergeb = StorageHelper.StorageLib.TypeConverter.FromDelegater[info.Value.PropertyType.Name](info.Value.GetValue(this, null));
										StateInfo.PropertiesContent[info.Key] = ergeb;
								}
						}
						else if (TypeInfo.PropertyInfo.ContainsKey(PropName))
						{
								PropertyInfo PInfo = TypeInfo.PropertyInfo[PropName];
								string Ergeb = StorageHelper.StorageLib.TypeConverter.FromDelegater[PInfo.PropertyType.Name](PInfo.GetValue(this, null));

								StateInfo.PropertiesContent[PropName] = Ergeb;
						}
				}

				private void UpdateStickyTrue(object sender, System.ComponentModel.PropertyChangedEventArgs e)
				{
						StateInfo.isSticky = true;
				}
		}
}