﻿/// <typ>StorageHelperType</typ>
/// <file>StorageHelperType.Static.cs</file>
/// <create>22.09.2011 05:59:37</crate>
/// <author>Wolfgang Stelzhammer</author>
///
/// <summary>
///
/// </summary>
/// <copyright>Copyright © 2008-2011 Wolfgang Stelzhammer, All rights reserved!</copyright>
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using StorageHelper.StorageLib;

namespace StorageHelper.StorageBase
{
		public partial class StorageHelperType
		{
				private static TypeInfo TypeInfo = new TypeInfo();

				public static void InitStatic(Type ReflectionType)
				{
						var Mothodes = typeof(StorageHelperType).GetMethods();
						var genericMethod = typeof(StorageHelperType).GetMethod("InitStatic", Type.EmptyTypes);
						var methodInstance = genericMethod.MakeGenericMethod(ReflectionType);
						methodInstance.Invoke(null, null);
				}

				public static void InitStatic<T>() where T : StorageHelperType
				{
						//#if DEBUG
						//      TypeInfo = new StorageLib.TypeInfo();
						//      TypeInfo.initedStatic = false;
						//#endif
						if (!TypeInfo.initedStatic && !inReadingAllContents)
						{
								TypeInfo.TypeReflection = typeof(T);
								StaticTypeInfo sti = getStaticTypeInfo();

								TypeInfo.TypeName = getTypeName();
								TypeInfo.TypeCollectionName = getTypeCollectionName();

								if (sti.IdDigits <= 0)
										TypeInfo.IdDigits = getIdDigitsAttribute();
								else
										TypeInfo.IdDigits = sti.IdDigits;

								if (sti.ConnectionString == null || sti.ConnectionString == string.Empty)
										TypeInfo.ConnectionString = getConnectionString();
								else
										TypeInfo.ConnectionString = sti.ConnectionString;

								TypeInfo.UniqIDs = getUniqIDs();

								TypeInfo.PropertyInfo = getProperties();

								TypeInfo.AutoSave = getAutosave();

								if (TypeInfo.Engine == null)
								{
										if (sti.StorageEngine == null)
												TypeInfo.Engine = getStorageEngine();
										else
												TypeInfo.Engine = sti.StorageEngine;

										TypeInfo.Engine.CustonConfig = sti.EngineCustomParameter;
										TypeInfo.Engine.Init(TypeInfo);
								}

								TypeInfo.initedStatic = true;
								Refresh();
						}
				}

				private static void Refresh()
				{
						TypeInfo.TypeContentsCollection = new ConcurrentObservableDictionary<string, TypeContent>();
						TypeInfo.Engine.GetAllContent(TypeInfo.TypeContentsCollection);
				}

				private static T ObjectInvoker<T>(string _uniqID) where T : StorageHelperType
				{
						T temp = Activator.CreateInstance<T>();
						TypeContent content = TypeInfo.TypeContentsCollection[_uniqID];
						foreach (var item in TypeInfo.PropertyInfo)
						{
								item.Value.SetValue(temp, TypeConverter.ToDelegater[item.Value.PropertyType.Name](content[item.Key]), null);
						}
						return temp;
				}

				private static bool inReadingAllContents = false;

				public static bool redingReady = true;

				#region static return Methods

				public static T returnObject<T>(string _uniqID) where T : StorageHelperType
				{
						InitStatic<T>();

						if (!TypeInfo.ObjectCollection.ContainsKey(_uniqID))
						{
								TypeInfo.ObjectCollection[_uniqID] = ObjectInvoker<T>(_uniqID);
						}
						return (T)TypeInfo.ObjectCollection[_uniqID];
				}

				public static void returnAllObjects<T>(List<T> TList) where T : StorageHelperType
				{
						InitStatic<T>();
						TList.Clear();

						foreach (var item in TypeInfo.TypeContentsCollection)
						{
								TList.Add(returnObject<T>(item.Key));
						}
				}

				public static void returnAllObjects<T>(Dictionary<string, T> tDict) where T : StorageHelperType
				{
						InitStatic<T>();
						tDict.Clear();

						foreach (var item in TypeInfo.TypeContentsCollection)
						{
								tDict.Add(item.Key, returnObject<T>(item.Key));
						}
				}

				public static void returnAllObjects<T>(ObservableDictionary<string, T> TDictionary) where T : StorageHelperType
				{
						InitStatic<T>();
						TDictionary.Clear();

						foreach (var item in TypeInfo.TypeContentsCollection)
						{
								TDictionary.Add(item.Key, returnObject<T>(item.Key));
						}
				}

				public static void returnAllObjects<T>(ObservableCollection<T> TDictionary) where T : StorageHelperType
				{
						InitStatic<T>();
						TDictionary.Clear();

						foreach (var item in TypeInfo.TypeContentsCollection)
						{
								TDictionary.Add(returnObject<T>(item.Key));
						}
				}

				public static void returnAllObjects<T>(ConcurrentObservableDictionary<string, T> TDictionary) where T : StorageHelperType
				{
						InitStatic<T>();
						TDictionary.Clear();

						foreach (var item in TypeInfo.TypeContentsCollection)
						{
								TDictionary.Add(item.Key, returnObject<T>(item.Key));
						}
				}

				public static void returnAllObjects<T>(BindingSource Bs) where T : StorageHelperType
				{
						InitStatic<T>();
						Bs.Clear();

						foreach (var item in TypeInfo.TypeContentsCollection)
						{
								Bs.Add(returnObject<T>(item.Key));
						}
				}

				#endregion static return Methods
		}
}