﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;

namespace StorageHelper.StorageLib
{
		public class CustomAttibute<T> : List<T>
		{
				private AttributeUsageAttribute Usage;

				public List<PropertyInfo> FoundProperties = new List<PropertyInfo>();
				public List<ConstructorInfo> FoundConstructors = new List<ConstructorInfo>();
				public List<EventInfo> FoundEvents = new List<EventInfo>();
				public List<FieldInfo> FoundFields = new List<FieldInfo>();
				public List<MethodInfo> FoundMethods = new List<MethodInfo>();

				public CustomAttibute(Type Object, Boolean Inherit = false, BindingFlags Bindings = BindingFlags.Default)
				{
						if (!typeof(T).IsSubclassOf(typeof(Attribute)))
								throw new ArgumentException("<T> muss von Attribute abgeleitet sein!");

						AttributeUsageAttribute[] use = (AttributeUsageAttribute[])typeof(T).GetCustomAttributes(typeof(AttributeUsageAttribute), Inherit);

						this.Clear();
						if (use.Count<AttributeUsageAttribute>() != 1)
								throw new ArgumentException("<T> muss ein AttributeUsageAttribute haben!!!");
						else
								Usage = use[0];

						if (Usage.ValidOn.HasFlag(AttributeTargets.Property))
						{
								PropertyInfo[] PIs;
								if (Bindings != BindingFlags.Default)
										PIs = Object.GetProperties(Bindings);
								else
										PIs = Object.GetProperties();
								foreach (PropertyInfo prop in PIs)
								{
										if (addToList(prop.GetCustomAttributes(typeof(T), Inherit)) > 0)
												FoundProperties.Add(prop);
								}
						}
						else if (Usage.ValidOn.HasFlag(AttributeTargets.Constructor))
						{
								ConstructorInfo[] PIs;
								if (Bindings != BindingFlags.Default)
										PIs = Object.GetConstructors(Bindings);
								else
										PIs = Object.GetConstructors();
								foreach (ConstructorInfo prop in PIs)
								{
										if (addToList(prop.GetCustomAttributes(typeof(T), Inherit)) > 0)
												FoundConstructors.Add(prop);
								}
						}
						else if (Usage.ValidOn.HasFlag(AttributeTargets.Event))
						{
								EventInfo[] PIs;
								if (Bindings != BindingFlags.Default)
										PIs = Object.GetEvents(Bindings);
								else
										PIs = Object.GetEvents();
								foreach (EventInfo prop in PIs)
								{
										if (addToList(prop.GetCustomAttributes(typeof(T), Inherit)) > 0)
												FoundEvents.Add(prop);
								}
						}
						else if (Usage.ValidOn.HasFlag(AttributeTargets.Field))
						{
								FieldInfo[] PIs;
								if (Bindings != BindingFlags.Default)
										PIs = Object.GetFields(Bindings);
								else
										PIs = Object.GetFields();

								foreach (FieldInfo prop in PIs)
								{
										if (addToList(prop.GetCustomAttributes(typeof(T), Inherit)) > 0)
												FoundFields.Add(prop);
								}
						}
						else if (Usage.ValidOn.HasFlag(AttributeTargets.Method))
						{
								MethodInfo[] PIs;
								if (Bindings != BindingFlags.Default)
										PIs = Object.GetMethods(Bindings);
								else
										PIs = Object.GetMethods();
								foreach (MethodInfo prop in PIs)
								{
										if (addToList(prop.GetCustomAttributes(typeof(T), Inherit)) > 0)
												FoundMethods.Add(prop);
								}
						}
						else if (Usage.ValidOn.HasFlag(AttributeTargets.Class))
								addToList(Object.GetCustomAttributes(typeof(T), Inherit));
				}

				private int addToList(object[] temp)
				{
						foreach (T Attrib in temp)
						{
								this.Add(Attrib);
						}
						return temp.Count<object>();
				}
		}
}